@@ -76,17 +76,25 @@ not work out-of-the-box.
 """
 import abc
 import base64
-import calendar
+import functools
 import json
 import logging
 from abc import ABC
 from datetime import datetime
 from email.utils import formatdate
-from typing import Optional, Union
+from typing import Optional, Tuple, Union
 from xml.etree import ElementTree as ETree
 
 from boto.utils import ISO8601
-from botocore.model import ListShape, MapShape, OperationModel, ServiceModel, Shape, StructureShape
+from botocore.model import (
+    ListShape,
+    MapShape,
+    NoShapeFoundError,
+    OperationModel,
+    ServiceModel,
+    Shape,
+    StructureShape,
+)
 from botocore.serialize import ISO8601_MICRO
 from botocore.utils import calculate_md5, is_json_value_header, parse_to_aware_datetime
 from moto.core.utils import gen_amzn_requestid_long
@@ -97,6 +105,55 @@ from localstack.utils.common import to_bytes, to_str
 LOG = logging.getLogger(__name__)
 
 
+class ResponseSerializerError(Exception):
+    """
+    Error which is thrown if the request serialization fails.
+    Super class of all exceptions raised by the serializer.
+    """
+
+    pass
+
+
+class UnknownSerializerError(ResponseSerializerError):
+    """
+    Error which indicates that the raised exception of the serializer could be caused by invalid data or by any other
+    (unknown) issue. Errors like this should be reported and indicate an issue in the serializer itself.
+    """
+
+    pass
+
+
+class ProtocolSerializerError(ResponseSerializerError):
+    """
+    Error which indicates that the given data is not compliant with the service's specification and cannot be serialized.
+    This usually results in a response to the client with an HTTP 5xx status code (internal server error).
+    """
+
+    pass
+
+
+def _handle_exceptions(func):
+    """
+    Decorator which handles the exceptions raised by the serializer. It ensures that all exceptions raised by the public
+    methods of the parser are instances of ResponseSerializerError.
+    :param func: to wrap in order to add the exception handling
+    :return: wrapped function
+    """
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        try:
+            return func(*args, **kwargs)
+        except ResponseSerializerError:
+            raise
+        except Exception as e:
+            raise UnknownSerializerError(
+                "An unknown error occurred when trying to serialize the response."
+            ) from e
+
+    return wrapper
+
+
 class ResponseSerializer(abc.ABC):
     """
     The response serializer is responsible for the serialization of a service implementation's result to an actual
@@ -108,6 +165,7 @@ class ResponseSerializer(abc.ABC):
     # The default timestamp format is ISO8601, but this can be overwritten by subclasses.
     TIMESTAMP_FORMAT = "iso8601"
 
+    @_handle_exceptions
     def serialize_to_response(
         self, response: dict, operation_model: OperationModel
     ) -> HttpResponse:
@@ -118,6 +176,7 @@ class ResponseSerializer(abc.ABC):
         :param operation_model: specification of the service & operation containing information about the shape of the
                                 service's output / response
         :return: HttpResponse which can be sent to the calling client
+        :raises: ResponseSerializerError (either a ProtocolSerializerError or an UnknownSerializerError)
         """
         serialized_response = self._create_default_response(operation_model)
         shape = operation_model.output_shape
@@ -131,6 +190,7 @@ class ResponseSerializer(abc.ABC):
         )
         return serialized_response
 
+    @_handle_exceptions
     def serialize_error_to_response(
         self, error: ServiceException, operation_model: OperationModel
     ) -> HttpResponse:
@@ -142,6 +202,7 @@ class ResponseSerializer(abc.ABC):
         :param operation_model: specification of the service & operation containing information about the shape of the
                                 service's output / response
         :return: HttpResponse which can be sent to the calling client
+        :raises: ResponseSerializerError (either a ProtocolSerializerError or an UnknownSerializerError)
         """
         serialized_response = self._create_default_response(operation_model)
         if isinstance(error, CommonServiceException):
@@ -154,15 +215,24 @@ class ResponseSerializer(abc.ABC):
             status_code = error.status_code
             shape = None
         else:
-            # It it's not a CommonServiceException, the exception is being serialized based on the specification
+            # It's not a CommonServiceException, the exception is being serialized based on the specification
 
             # The shape name is equal to the class name (since the classes are generated from the shape's name)
             error_shape_name = error.__class__.__name__
-
             # Lookup the corresponding error shape in the operation model
-            shape = next(
-                shape for shape in operation_model.error_shapes if shape.name == error_shape_name
-            )
+            try:
+                shape = operation_model.service_model.shape_for(error_shape_name)
+                if not shape.metadata.get("exception", False):
+                    raise ProtocolSerializerError(
+                        f"The given error ({error_shape_name}) corresponds to a non-exception"
+                        f"shape."
+                    )
+            except NoShapeFoundError as e:
+                raise ProtocolSerializerError(
+                    f"Error to serialize ({error_shape_name}) neither is a CommonServiceException, nor is its error "
+                    f"shape contained in the service's specification ({operation_model.service_model.service_name})."
+                ) from e
+
             error_spec = shape.metadata.get("error", {})
             status_code = error_spec.get("httpStatusCode")
 
@@ -242,7 +312,7 @@ class ResponseSerializer(abc.ABC):
 
     @staticmethod
     def _timestamp_unixtimestamp(value: datetime) -> float:
-        return round(value.timestamp(), 3)
+        return value.timestamp()
 
     def _timestamp_rfc822(self, value: datetime) -> str:
         if isinstance(value, datetime):
@@ -288,17 +358,17 @@ class ResponseSerializer(abc.ABC):
     ):
         """Applies additional traits on the raw response for a given model or protocol."""
         if operation_model.http_checksum_required:
-            self.add_md5_header(response)
+            self._add_md5_header(response)
         return response
 
-    def has_header(self, header_name: str, headers: dict):
+    def _has_header(self, header_name: str, headers: dict):
         """Case-insensitive check for header key."""
         if header_name is None:
             return False
         else:
             return header_name.lower() in [key.lower() for key in headers.keys()]
 
-    def add_md5_header(self, response: HttpResponse):
+    def _add_md5_header(self, response: HttpResponse):
         """Add a Content-MD5 header if not yet there. Adapted from botocore.utils"""
         headers = response.headers
         body = response.data
@@ -306,6 +376,9 @@ class ResponseSerializer(abc.ABC):
             md5_digest = calculate_md5(body)
             headers["Content-MD5"] = md5_digest
 
+    def _get_error_message(self, error: Exception) -> Optional[str]:
+        return str(error) if error is not None and str(error) != "None" else None
+
 
 class BaseXMLResponseSerializer(ResponseSerializer):
     """
@@ -347,12 +420,12 @@ class BaseXMLResponseSerializer(ResponseSerializer):
     ) -> None:
         code_tag = ETree.SubElement(error_tag, "Code")
         code_tag.text = code
-        message = str(error)
-        if len(message) > 0:
+        message = self._get_error_message(error)
+        if message:
             self._default_serialize(error_tag, message, None, "Message")
         if sender_fault:
             # The sender fault is either not set or "Sender"
-            self._default_serialize(error_tag, "Sender", None, "Fault")
+            self._default_serialize(error_tag, "Sender", None, "Type")
 
     def _serialize_body_params(
         self, params: dict, shape: Shape, operation_model: OperationModel
@@ -550,8 +623,12 @@ class BaseRestResponseSerializer(ResponseSerializer, ABC):
         shape_members: dict,
         operation_model: OperationModel,
     ) -> None:
-        self._process_header_members(parameters, response, shape)
-        self._serialize_payload(parameters, response, shape, shape_members, operation_model)
+        header_params, payload_params = self._partition_members(parameters, shape)
+        self._process_header_members(header_params, response, shape)
+        # "HEAD" responeses are basically "GET" responses without the actual body.
+        # Do not process the body payload in this case (setting a body could also manipulate the headers)
+        if operation_model.http.get("method") != "HEAD":
+            self._serialize_payload(payload_params, response, shape, shape_members, operation_model)
         self._serialize_content_type(response, shape, shape_members)
         self._prepare_additional_traits_in_response(response, operation_model)
 
@@ -632,12 +709,15 @@ class BaseRestResponseSerializer(ResponseSerializer, ABC):
                 continue
             key = member_shape.serialization.get("name", name)
             value = parameters[name]
+            if value is None:
+                continue
             if location == "header":
                 response.headers[key] = self._serialize_header_value(member_shape, value)
             elif location == "headers":
                 header_prefix = key
                 self._serialize_header_map(header_prefix, response, value)
-            del parameters[name]
+            elif location == "statusCode":
+                response.status_code = int(value)
 
     def _serialize_header_map(self, prefix: str, response: HttpResponse, params: dict) -> None:
         """Serializes the header map for the location trait "headers"."""
@@ -649,16 +729,18 @@ class BaseRestResponseSerializer(ResponseSerializer, ABC):
         """Serializes a value for the location trait "header"."""
         if shape.type_name == "timestamp":
             datetime_obj = parse_to_aware_datetime(value)
-            timestamp = calendar.timegm(datetime_obj.utctimetuple())
             timestamp_format = shape.serialization.get(
                 "timestampFormat", self.HEADER_TIMESTAMP_FORMAT
             )
-            return self._convert_timestamp_to_str(timestamp, timestamp_format)
+            return self._convert_timestamp_to_str(datetime_obj, timestamp_format)
         elif shape.type_name == "list":
             converted_value = [
                 self._serialize_header_value(shape.member, v) for v in value if v is not None
             ]
             return ",".join(converted_value)
+        elif shape.type_name == "boolean":
+            # Set the header value to "true" if the given value is truthy, otherwise set the header value to "false".
+            return "true" if value else "false"
         elif is_json_value_header(shape):
             # Serialize with no spaces after separators to save space in
             # the header.
@@ -666,6 +748,26 @@ class BaseRestResponseSerializer(ResponseSerializer, ABC):
         else:
             return value
 
+    def _partition_members(self, parameters: dict, shape: Optional[Shape]) -> Tuple[dict, dict]:
+        """Separates the top-level keys in the given parameters dict into header- and payload-located params."""
+        if not isinstance(shape, StructureShape):
+            # If the shape isn't a structure, we default to the whole response being parsed in the body.
+            # Non-payload members are only loated in the top-level hierarchy and those are always structures.
+            return {}, parameters
+        header_params = {}
+        payload_params = {}
+        shape_members = shape.members
+        for name in shape_members:
+            member_shape = shape_members[name]
+            if name not in parameters:
+                continue
+            location = member_shape.serialization.get("location")
+            if location:
+                header_params[name] = parameters[name]
+            else:
+                payload_params[name] = parameters[name]
+        return header_params, payload_params
+
 
 class RestXMLResponseSerializer(BaseRestResponseSerializer, BaseXMLResponseSerializer):
     """
@@ -812,10 +914,11 @@ class EC2ResponseSerializer(QueryResponseSerializer):
     def _prepare_additional_traits_in_xml(self, root: Optional[ETree.Element]):
         # The EC2 protocol does not use the root output shape, therefore we need to remove the hierarchy level
         # below the root level
-        output_node = root[0]
-        for child in output_node:
-            root.append(child)
-        root.remove(output_node)
+        if len(root) > 0:
+            output_node = root[0]
+            for child in output_node:
+                root.append(child)
+            root.remove(output_node)
 
         # Add the requestId here (it's not defined in the specs)
         # For the ec2 and the query protocol, the root cannot be None at this time.
@@ -845,7 +948,10 @@ class JSONResponseSerializer(ResponseSerializer):
         operation_model: OperationModel,
     ) -> None:
         # TODO handle error shapes with members
-        body = {"__type": code, "message": str(error)}
+        body = {"__type": code}
+        message = self._get_error_message(error)
+        if message is not None:
+            body["message"] = message
         response.set_json(body)
 
     def _serialize_response(
@@ -966,7 +1072,7 @@ class RestJSONResponseSerializer(BaseRestResponseSerializer, JSONResponseSeriali
             return
 
         has_body = serialized.data != b""
-        has_content_type = self.has_header("Content-Type", serialized.headers)
+        has_content_type = self._has_header("Content-Type", serialized.headers)
         if has_body and not has_content_type:
             serialized.headers["Content-Type"] = "application/json"
 
