@@ -1,11 +1,16 @@
+from localstack.aws.proxy import AwsApiListener
+from localstack.services.moto import MotoFallbackDispatcher
 from localstack.services.plugins import Service, aws_provider
 
 
 @aws_provider()
 def acm():
-    from localstack.services.acm import acm_starter
+    from localstack.services.acm.provider import AcmProvider
+    from localstack.services.moto import MotoFallbackDispatcher
 
-    return Service("acm", start=acm_starter.start_acm)
+    provider = AcmProvider()
+
+    return Service("acm", listener=AwsApiListener("acm", MotoFallbackDispatcher(provider)))
 
 
 @aws_provider()
@@ -28,19 +33,25 @@ def cloudformation():
 
 @aws_provider(api="config")
 def awsconfig():
-    from localstack.services.configservice import configservice_starter
+    from localstack.services.configservice.provider import ConfigProvider
+    from localstack.services.moto import MotoFallbackDispatcher
 
-    return Service("config", start=configservice_starter.start_configservice)
+    provider = ConfigProvider()
+    return Service("config", listener=AwsApiListener("config", MotoFallbackDispatcher(provider)))
 
 
 @aws_provider()
 def cloudwatch():
-    from localstack.services.cloudwatch import cloudwatch_listener, cloudwatch_starter
+    from localstack.services.cloudwatch.provider import CloudwatchProvider
+    from localstack.services.moto import MotoFallbackDispatcher
+
+    provider = CloudwatchProvider()
+    listener = AwsApiListener("cloudwatch", MotoFallbackDispatcher(provider))
 
     return Service(
         "cloudwatch",
-        listener=cloudwatch_listener.UPDATE_CLOUD_WATCH,
-        start=cloudwatch_starter.start_cloudwatch,
+        listener=listener,
+        lifecycle_hook=provider,
     )
 
 
@@ -58,9 +69,15 @@ def dynamodb():
 
 @aws_provider()
 def dynamodbstreams():
-    from localstack.services.dynamodbstreams import dynamodbstreams_starter
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.dynamodbstreams.provider import DynamoDBStreamsProvider
 
-    return Service("dynamodbstreams", start=dynamodbstreams_starter.start_dynamodbstreams)
+    provider = DynamoDBStreamsProvider()
+    return Service(
+        "dynamodbstreams",
+        listener=AwsApiListener("dynamodbstreams", provider),
+        lifecycle_hook=provider,
+    )
 
 
 @aws_provider()
@@ -72,9 +89,11 @@ def ec2():
 
 @aws_provider()
 def es():
-    from localstack.services.es import es_starter
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.es.provider import EsProvider
 
-    return Service("es", start=es_starter.start_elasticsearch_service)
+    provider = EsProvider()
+    return Service("es", listener=AwsApiListener("es", provider))
 
 
 @aws_provider()
@@ -151,9 +170,12 @@ def opensearch():
 
 @aws_provider()
 def redshift():
-    from localstack.services.redshift import redshift_starter
+    from localstack.services.redshift.provider import RedshiftProvider
 
-    return Service("redshift", start=redshift_starter.start_redshift)
+    provider = RedshiftProvider()
+    listener = AwsApiListener("redshift", MotoFallbackDispatcher(provider))
+
+    return Service("redshift", listener=listener)
 
 
 @aws_provider()
@@ -231,9 +253,14 @@ def sqs_asf():
 
 @aws_provider()
 def ssm():
-    from localstack.services.ssm import ssm_listener, ssm_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.ssm.provider import SsmProvider
 
-    return Service("ssm", listener=ssm_listener.UPDATE_SSM, start=ssm_starter.start_ssm)
+    provider = SsmProvider()
+    return Service(
+        "ssm",
+        listener=AwsApiListener("ssm", MotoFallbackDispatcher(provider)),
+    )
 
 
 @aws_provider()
@@ -259,34 +286,49 @@ def stepfunctions():
 
 @aws_provider()
 def swf():
-    from localstack.services.swf import swf_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.swf.provider import SWFProvider
 
+    provider = SWFProvider()
     return Service(
         "swf",
-        start=swf_starter.start_swf,
+        listener=AwsApiListener("swf", MotoFallbackDispatcher(provider)),
     )
 
 
 @aws_provider()
 def resourcegroupstaggingapi():
-    from localstack.services.resourcegroupstaggingapi import rgta_listener, rgta_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.resourcegroupstaggingapi.provider import (
+        ResourcegroupstaggingapiProvider,
+    )
 
+    provider = ResourcegroupstaggingapiProvider()
     return Service(
         "resourcegroupstaggingapi",
-        listener=rgta_listener.UPDATE_RGSA,
-        start=rgta_starter.start_rgsa,
+        listener=AwsApiListener("resourcegroupstaggingapi", MotoFallbackDispatcher(provider)),
     )
 
 
 @aws_provider(api="resource-groups")
 def resource_groups():
-    from localstack.services.resourcegroups import rg_listener, rg_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.resourcegroups.provider import ResourceGroupsProvider
 
-    return Service("resource-groups", listener=rg_listener.UPDATE_RG, start=rg_starter.start_rg)
+    provider = ResourceGroupsProvider()
+    return Service(
+        "resource-groups",
+        listener=AwsApiListener("resource-groups", MotoFallbackDispatcher(provider)),
+    )
 
 
 @aws_provider()
 def support():
-    from localstack.services.support import support_starter
+    from localstack.services.moto import MotoFallbackDispatcher
+    from localstack.services.support.provider import SupportProvider
 
-    return Service("support", start=support_starter.start_support)
+    provider = SupportProvider()
+    return Service(
+        "support",
+        listener=AwsApiListener("support", MotoFallbackDispatcher(provider)),
+    )
